import { PostgrestQueryBuilder, PostgrestSingleResponse, PostgrestMaybeSingleResponse, PostgrestResponse, PostgrestFilterBuilder, PostgrestTransformBuilder } from '@supabase/postgrest-js';
import { GetResult } from '@supabase/postgrest-js/dist/module/select-query-parser';
import { GenericSchema, GenericTable } from '@supabase/postgrest-js/dist/module/types';
import { GenericSchema as GenericSchema$1 } from '@supabase/supabase-js/dist/module/lib/types';

declare const buildDeleteFetcher: <S extends GenericSchema, T extends GenericTable, Q extends string = "*", R = GetResult<S, T["Row"], Q extends "*" ? "*" : Q>>(qb: PostgrestQueryBuilder<S, T>, primaryKeys: (keyof T["Row"])[], query?: Q | undefined) => (input: Partial<T["Row"]>) => Promise<R>;

declare type FetcherType = "single" | "maybeSingle" | "multiple";
declare type PostgrestFetcherResponse<Type> = Pick<PostgrestSingleResponse<Type>, "data"> | Pick<PostgrestMaybeSingleResponse<Type>, "data"> | Pick<PostgrestResponse<Type>, "data" | "count">;
declare function createFetcher<Schema extends GenericSchema, Row extends Record<string, unknown>, Result>(mode: FetcherType): (query: PostgrestFilterBuilder<Schema, Row, Result>) => Promise<PostgrestFetcherResponse<Result>>;

declare const buildInsertFetcher: <S extends GenericSchema$1, T extends GenericTable, Q extends string = "*", R = GetResult<S, T["Row"], Q extends "*" ? "*" : Q>>(qb: PostgrestQueryBuilder<S, T>, mode: "single" | "multiple", query?: Q | undefined) => (input: T["Insert"] | T["Insert"][]) => Promise<R | R[]>;

declare type PostgrestPaginationFetcher<Type, Args extends any[]> = (...args: Args) => Promise<Type[] | null>;
declare type PostgrestPaginationKeyDecoder<Args extends any[]> = (...args: Args) => {
    limit?: number;
    offset?: number;
};
declare const createPaginationFetcher: <Schema extends GenericSchema$1, Row extends Record<string, unknown>, Result, Args extends any[]>(query: PostgrestTransformBuilder<Schema, Row, Result> | null, decode: PostgrestPaginationKeyDecoder<Args>, pageSize: number) => PostgrestPaginationFetcher<Result, Args> | null;
declare const createPaginationHasMoreFetcher: <Schema extends GenericSchema$1, Row extends Record<string, unknown>, Result, Args extends any[]>(query: PostgrestTransformBuilder<Schema, Row, Result> | null, decode: PostgrestPaginationKeyDecoder<Args>, pageSize: number) => PostgrestPaginationFetcher<Result | {
    hasMore: true;
}, Args> | null;

declare const buildUpdateFetcher: <S extends GenericSchema, T extends GenericTable, Q extends string = "*", R = GetResult<S, T["Row"], Q extends "*" ? "*" : Q>>(qb: PostgrestQueryBuilder<S, T>, primaryKeys: (keyof T["Row"])[], query?: Q | undefined) => (input: Partial<T["Row"]>) => Promise<R>;

declare const buildUpsertFetcher: <S extends GenericSchema$1, T extends GenericTable, Q extends string = "*", R = GetResult<S, T["Row"], Q extends "*" ? "*" : Q>>(qb: PostgrestQueryBuilder<S, T>, mode: "single" | "multiple", query?: Q | undefined) => (input: T["Insert"] | T["Insert"][]) => Promise<R | R[]>;

export { FetcherType, PostgrestFetcherResponse, PostgrestPaginationFetcher, PostgrestPaginationKeyDecoder, buildDeleteFetcher, buildInsertFetcher, buildUpdateFetcher, buildUpsertFetcher, createFetcher, createPaginationFetcher, createPaginationHasMoreFetcher };
